local addonName, addon = ...

local After, UIParent = C_Timer.After, UIParent

local LDA = LibStub('LibDisplayAssist-1.4')
local CalculateY, GetScaledRect = LDA._CalculateY, LDA._GetScaledRect

local registry, DISPLAY_WIDTH, DISPLAY_HEIGHT, HALF_DISPLAY_HEIGHT = { }

local RepositionUIParent = addon.DoNothing										-- Just to be safe

--[[----------------------------------------------------------------------------
Jostle
------------------------------------------------------------------------------]]
local Refresh
do
	local refreshing

	local function Reposition()
		refreshing = nil
		local offsetBottom, offsetTop = 0, 0
		for frame in pairs(registry) do
			local _, bottom, _, height = GetScaledRect(frame)
			if bottom + height * 0.5 >= 0.5 then					-- Top
				local offset = bottom - 1
				if offset < offsetTop then
					offsetTop = offset
				end
			else													-- Bottom
				local top = bottom + height
				if top > offsetBottom then
					offsetBottom = top
				end
			end
		end
		RepositionUIParent('TOPLEFT', CalculateY(UIParent, offsetTop))
		RepositionUIParent('BOTTOMRIGHT', CalculateY(UIParent, offsetBottom))
	end

	function Refresh()
		if refreshing then return end
		refreshing = true
		After(0, Reposition)
	end
end

addon.Jostle = {
	["Refresh"] = Refresh,

	["Register"] = function(frame)
		if not registry[frame] then
			registry[frame] = true
			Refresh()
		end
	end,

	["Unregister"] = function(frame)
		if registry[frame] then
			registry[frame] = nil
			Refresh()
		end
	end
}

--[[----------------------------------------------------------------------------
Update with display size changes
------------------------------------------------------------------------------]]
local function OnSizeChanged(_, _, width, height)
	HALF_DISPLAY_HEIGHT, DISPLAY_HEIGHT, DISPLAY_WIDTH = height * 0.5, height, width
	if addon.settings then
		addon:AllPanels("Refresh")
	end
end

LDA.RegisterCallback(addon, 'DISPLAY_SIZE_CHANGED', OnSizeChanged)

LDA.RegisterCallback(addon, 'PLAYER_LOGOUT', function()
	addon:AllPanels("SaveOffset")
end)

OnSizeChanged(nil, nil, LDA.GetDisplaySize())

--[[----------------------------------------------------------------------------
Tweaks
------------------------------------------------------------------------------]]
if InterfaceOptionsFrame then													-- Keep centered on screen
	local strata, level = InterfaceOptionsFrame:GetFrameStrata(), InterfaceOptionsFrame:GetFrameLevel()
	InterfaceOptionsFrame:ClearAllPoints()
	InterfaceOptionsFrame:SetPoint('CENTER', nil, 'CENTER', 0, 0)
	InterfaceOptionsFrame:SetFrameStrata(strata)
	InterfaceOptionsFrame:SetFrameLevel(level)
end

if UpdateUIParentRelativeToDebugMenu then										-- Changes UIParent TOPLEFT
	hooksecurefunc("UpdateUIParentRelativeToDebugMenu", Refresh)
end

if WorldMapFrame and WorldMapFrame.HandleUserActionMaximizeSelf then			-- Classic compatibility
	hooksecurefunc(WorldMapFrame, 'HandleUserActionMaximizeSelf', function()
		addon:AllPanels("AdjustForWorldMapFrame", true)
		RepositionUIParent('TOPLEFT', 0)
		RepositionUIParent('BOTTOMRIGHT', 0)
	end)

	hooksecurefunc(WorldMapFrame, 'HandleUserActionMinimizeSelf', function()
		addon:AllPanels("AdjustForWorldMapFrame", false)
		Refresh()
	end)
end

--[[----------------------------------------------------------------------------
RepositionUIParent
------------------------------------------------------------------------------]]
do
	local isSafe, queue = not InCombatLockdown(), { }

	addon:RegisterEvent('PLAYER_REGEN_DISABLED', function()
		isSafe = false
	end)

	addon:RegisterEvent('PLAYER_REGEN_ENABLED', function()
		isSafe = true
		for point, offset in pairs(queue) do
			queue[point] = nil
			UIParent:SetPoint(point, 0, offset)
		end
	end)

	RepositionUIParent = function(point, offset)
		if isSafe then
			UIParent:SetPoint(point, 0, offset)
		else
			queue[point] = offset
		end
	end
end

--[[----------------------------------------------------------------------------
Moved here from Core due to HALF_DISPLAY_HEIGHT
------------------------------------------------------------------------------]]
function addon.GetAnchorInfo(frame)
	local _, frameCenter = frame:GetCenter()
	local scale = frame:GetEffectiveScale()
	if frameCenter * scale >= HALF_DISPLAY_HEIGHT then
		return 'TOP', frame, 'BOTTOM', 0, -1
	end
	return 'BOTTOM', frame, 'TOP', 0, 1
end
